@@ -1,5 +1,5 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (c) 2003 Hewlett-Packard Development Company, L.P.
+   Copyright (c) 2003-2005 Hewlett-Packard Development Company, L.P.
 	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
@@ -26,8 +26,15 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 #ifndef dwarf_h
 #define dwarf_h
 
-#include "internal.h"
-#include "mempool.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef HAVE_ATOMIC_OPS_H
+# include <atomic_ops.h>
+#endif
+
+#include <libunwind.h>
 
 struct dwarf_cursor;	/* forward-declaration */
 
@@ -145,7 +152,7 @@ typedef enum
     DW_CFA_def_cfa_register	= 0x0d,
     DW_CFA_def_cfa_offset	= 0x0e,
     DW_CFA_def_cfa_expression	= 0x0f,
-    DW_CFA_CFA_expression	= 0x10,
+    DW_CFA_expression		= 0x10,
     DW_CFA_offset_extended_sf	= 0x11,
     DW_CFA_def_cfa_sf		= 0x12,
     DW_CFA_def_cfa_offset_sf	= 0x13,
@@ -199,6 +206,7 @@ dwarf_cfa_t;
 #define DW_EH_PE_aligned	0x50	/* aligned pointer */
 
 extern struct mempool dwarf_reg_state_pool;
+extern struct mempool dwarf_cie_info_pool;
 
 typedef enum
   {
@@ -234,9 +242,33 @@ typedef struct dwarf_reg_state
   {
     struct dwarf_reg_state *next;	/* for rs_stack */
     dwarf_save_loc_t reg[DWARF_NUM_PRESERVED_REGS + 2];
+    unw_word_t ip;		          /* ip this rs is for */
+    unw_word_t ret_addr_column;           /* indicates which column in the rule table represents return address */
+    unsigned short lru_chain;	  /* used for least-recently-used chain */
+    unsigned short coll_chain;	/* used for hash collisions */
+    unsigned short hint;	      /* hint for next rs to try (or -1) */
   }
 dwarf_reg_state_t;
 
+typedef struct dwarf_cie_info
+  {
+    unw_word_t cie_instr_start;	/* start addr. of CIE "initial_instructions" */
+    unw_word_t cie_instr_end;	/* end addr. of CIE "initial_instructions" */
+    unw_word_t fde_instr_start;	/* start addr. of FDE "instructions" */
+    unw_word_t fde_instr_end;	/* end addr. of FDE "instructions" */
+    unw_word_t code_align;	/* code-alignment factor */
+    unw_word_t data_align;	/* data-alignment factor */
+    unw_word_t ret_addr_column;	/* column of return-address register */
+    unw_word_t handler;		/* address of personality-routine */
+    uint16_t abi;
+    uint16_t tag;
+    uint8_t fde_encoding;
+    uint8_t lsda_encoding;
+    unsigned int sized_augmentation : 1;
+    unsigned int have_abi_marker : 1;
+  }
+dwarf_cie_info_t;
+
 typedef struct dwarf_state_record
   {
     unsigned char fde_encoding;
@@ -256,16 +288,64 @@ typedef struct dwarf_cursor
     unw_word_t ip;		/* instruction pointer */
     unw_word_t args_size;	/* size of arguments */
     unw_word_t ret_addr_column;	/* column for return-address */
+    unw_word_t eh_args[UNW_TDEP_NUM_EH_REGS];
+    unsigned int eh_valid_mask;
 
     dwarf_loc_t loc[DWARF_NUM_PRESERVED_REGS];
 
     unsigned int pi_valid :1;	/* is proc_info valid? */
     unsigned int pi_is_dynamic :1; /* proc_info found via dynamic proc info? */
-    unsigned int cfa_is_sp :1;	/* TRUE if stack-pointer is CFA */
     unw_proc_info_t pi;		/* info about current procedure */
+
+    short hint; /* faster lookup of the rs cache */
+    short prev_rs;
   }
 dwarf_cursor_t;
 
+#define DWARF_LOG_UNW_CACHE_SIZE	7
+#define DWARF_UNW_CACHE_SIZE	(1 << DWARF_LOG_UNW_CACHE_SIZE)
+
+#define DWARF_LOG_UNW_HASH_SIZE	(DWARF_LOG_UNW_CACHE_SIZE + 1)
+#define DWARF_UNW_HASH_SIZE	(1 << DWARF_LOG_UNW_HASH_SIZE)
+
+typedef unsigned char unw_hash_index_t;
+
+struct dwarf_rs_cache
+  {
+#ifdef HAVE_ATOMIC_OPS_H
+    AO_TS_t busy;		/* is the rs-cache busy? */
+#else
+    pthread_mutex_t lock;
+#endif
+    unsigned short lru_head;	/* index of lead-recently used rs */
+    unsigned short lru_tail;	/* index of most-recently used rs */
+
+    /* hash table that maps instruction pointer to rs index: */
+    unsigned short hash[DWARF_UNW_HASH_SIZE];
+
+    uint32_t generation;	/* generation number */
+
+    /* rs cache: */
+    dwarf_reg_state_t buckets[DWARF_UNW_CACHE_SIZE];
+  };
+
+/* A list of descriptors for loaded .debug_frame sections.  */
+
+struct unw_debug_frame_list
+  {
+    /* The start (inclusive) and end (exclusive) of the described region.  */
+    unw_word_t start;
+    unw_word_t end;
+    /* The debug frame itself.  */
+    char *debug_frame;
+    size_t debug_frame_size;
+    /* Index (for binary search).  */
+    struct table_entry *index;
+    size_t index_size;
+    /* Pointer to next descriptor.  */
+    struct unw_debug_frame_list *next;
+  };
+
 /* Convenience macros: */
 #define dwarf_init			UNW_ARCH_OBJ (dwarf_init)
 #define dwarf_find_proc_info		UNW_OBJ (dwarf_find_proc_info)
@@ -273,7 +353,8 @@ dwarf_cursor_t;
 #define dwarf_put_unwind_info		UNW_OBJ (dwarf_put_unwind_info)
 #define dwarf_put_unwind_info		UNW_OBJ (dwarf_put_unwind_info)
 #define dwarf_eval_expr			UNW_OBJ (dwarf_eval_expr)
-#define dwarf_parse_fde			UNW_OBJ (dwarf_parse_fde)
+#define dwarf_extract_proc_info_from_fde \
+		UNW_OBJ (dwarf_extract_proc_info_from_fde)
 #define dwarf_find_save_locs		UNW_OBJ (dwarf_find_save_locs)
 #define dwarf_create_state_record	UNW_OBJ (dwarf_create_state_record)
 #define dwarf_make_proc_info		UNW_OBJ (dwarf_make_proc_info)
@@ -294,9 +375,13 @@ extern void dwarf_put_unwind_info (unw_addr_space_t as,
 extern int dwarf_eval_expr (struct dwarf_cursor *c, unw_word_t *addr,
 			    unw_word_t len, unw_word_t *valp,
 			    int *is_register);
-extern int dwarf_parse_fde (unw_addr_space_t as, unw_accessors_t *a,
-			    unw_word_t *addr, unw_proc_info_t *pi,
-			    unw_dyn_dwarf_fde_info_t *dfi, void *arg);
+extern int dwarf_extract_proc_info_from_fde (unw_addr_space_t as,
+					     unw_accessors_t *a,
+					     unw_word_t *fde_addr,
+					     unw_proc_info_t *pi,
+					     int need_unwind_info,
+					     unw_word_t base,
+					     void *arg);
 extern int dwarf_find_save_locs (struct dwarf_cursor *c);
 extern int dwarf_create_state_record (struct dwarf_cursor *c,
 				      dwarf_state_record_t *sr);
@@ -305,7 +390,7 @@ extern int dwarf_read_encoded_pointer (unw_addr_space_t as,
 				       unw_accessors_t *a,
 				       unw_word_t *addr,
 				       unsigned char encoding,
-				       unw_proc_info_t *pi,
+				       const unw_proc_info_t *pi,
 				       unw_word_t *valp, void *arg);
 extern int dwarf_step (struct dwarf_cursor *c);
 
