@@ -46,8 +46,13 @@ extern "C" {
 #define UNW_TDEP_CURSOR_LEN	127
 
 typedef uint32_t unw_word_t;
+typedef int32_t unw_sword_t;
 
-typedef long double unw_tdep_fpreg_t;
+typedef union {
+  struct { uint8_t b[4]; } val32;
+  struct { uint8_t b[10]; } val80;
+  struct { uint8_t b[16]; } val128;
+} unw_tdep_fpreg_t;
 
 typedef enum
   {
@@ -68,7 +73,7 @@ typedef enum
        wouldn't be with the DWARF numbering.  */
     UNW_X86_EAX,	/* scratch (exception argument 1) */
     UNW_X86_EDX,	/* scratch (exception argument 2) */
-    UNW_X86_ECX,	/* preserved */
+    UNW_X86_ECX,	/* scratch */
     UNW_X86_EBX,	/* preserved */
     UNW_X86_ESI,	/* preserved */
     UNW_X86_EDI,	/* preserved */
@@ -130,7 +135,16 @@ typedef enum
     /* frame info (read-only) */
     UNW_X86_CFA,
 
-    UNW_TDEP_LAST_REG = UNW_X86_LDT,
+    UNW_X86_XMM0,	/* scratch */
+    UNW_X86_XMM1,	/* scratch */
+    UNW_X86_XMM2,	/* scratch */
+    UNW_X86_XMM3,	/* scratch */
+    UNW_X86_XMM4,	/* scratch */
+    UNW_X86_XMM5,	/* scratch */
+    UNW_X86_XMM6,	/* scratch */
+    UNW_X86_XMM7,	/* scratch */
+
+    UNW_TDEP_LAST_REG = UNW_X86_XMM7,
 
     UNW_TDEP_IP = UNW_X86_EIP,
     UNW_TDEP_SP = UNW_X86_CFA,
@@ -159,7 +173,7 @@ typedef ucontext_t unw_tdep_context_t;
 
 typedef struct
   {
-    unw_dyn_dwarf_fde_info_t dwarf_info;
+    /* no x86-specific auxiliary proc-info */
   }
 unw_tdep_proc_info_t;
 
