@@ -1,5 +1,5 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (c) 2003 Hewlett-Packard Development Company, L.P.
+   Copyright (c) 2003, 2005 Hewlett-Packard Development Company, L.P.
 	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
@@ -23,10 +23,9 @@ LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
-#include <string.h>
-
+#include <stddef.h>
 #include "dwarf_i.h"
-#include "tdep.h"
+#include "libunwind_i.h"
 
 #define alloc_reg_state()	(mempool_alloc (&dwarf_reg_state_pool))
 #define free_reg_state(rs)	(mempool_free (&dwarf_reg_state_pool, rs))
@@ -60,7 +59,7 @@ set_reg (dwarf_state_record_t *sr, unw_word_t regnum, dwarf_where_t where,
 static int
 run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 		 unw_word_t ip, unw_word_t *addr, unw_word_t end_addr,
-		 unw_dyn_dwarf_fde_info_t *dfi)
+		 struct dwarf_cie_info *dci)
 {
   unw_word_t curr_ip, operand = 0, regnum, val, len, fde_encoding;
   dwarf_reg_state_t *rs_stack = NULL, *new_rs, *old_rs;
@@ -90,28 +89,28 @@ run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
       switch ((dwarf_cfa_t) op)
 	{
 	case DW_CFA_advance_loc:
-	  curr_ip += operand * dfi->code_align;
+	  curr_ip += operand * dci->code_align;
 	  Debug (15, "CFA_advance_loc to 0x%lx\n", (long) curr_ip);
 	  break;
 
 	case DW_CFA_advance_loc1:
 	  if ((ret = dwarf_readu8 (as, a, addr, &u8, arg)) < 0)
 	    goto fail;
-	  curr_ip += u8 * dfi->code_align;
+	  curr_ip += u8 * dci->code_align;
 	  Debug (15, "CFA_advance_loc1 to 0x%lx\n", (long) curr_ip);
 	  break;
 
 	case DW_CFA_advance_loc2:
 	  if ((ret = dwarf_readu16 (as, a, addr, &u16, arg)) < 0)
 	    goto fail;
-	  curr_ip += u16 * dfi->code_align;
+	  curr_ip += u16 * dci->code_align;
 	  Debug (15, "CFA_advance_loc2 to 0x%lx\n", (long) curr_ip);
 	  break;
 
 	case DW_CFA_advance_loc4:
 	  if ((ret = dwarf_readu32 (as, a, addr, &u32, arg)) < 0)
 	    goto fail;
-	  curr_ip += u32 * dfi->code_align;
+	  curr_ip += u32 * dci->code_align;
 	  Debug (15, "CFA_advance_loc4 to 0x%lx\n", (long) curr_ip);
 	  break;
 
@@ -122,7 +121,7 @@ run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 
 	    if ((ret = dwarf_readu64 (as, a, addr, &u64, arg)) < 0)
 	      goto fail;
-	    curr_ip += u64 * dfi->code_align;
+	    curr_ip += u64 * dci->code_align;
 	    Debug (15, "CFA_MIPS_advance_loc8\n");
 	    break;
 	  }
@@ -143,27 +142,27 @@ run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 	    }
 	  if ((ret = dwarf_read_uleb128 (as, a, addr, &val, arg)) < 0)
 	    goto fail;
-	  set_reg (sr, regnum, DWARF_WHERE_CFAREL, val * dfi->data_align);
+	  set_reg (sr, regnum, DWARF_WHERE_CFAREL, val * dci->data_align);
 	  Debug (15, "CFA_offset r%lu at cfa+0x%lx\n",
-		 (long) regnum, (long) (val * dfi->data_align));
+		 (long) regnum, (long) (val * dci->data_align));
 	  break;
 
 	case DW_CFA_offset_extended:
 	  if (((ret = read_regnum (as, a, addr, &regnum, arg)) < 0)
 	      || ((ret = dwarf_read_uleb128 (as, a, addr, &val, arg)) < 0))
 	    goto fail;
-	  set_reg (sr, regnum, DWARF_WHERE_CFAREL, val * dfi->data_align);
+	  set_reg (sr, regnum, DWARF_WHERE_CFAREL, val * dci->data_align);
 	  Debug (15, "CFA_offset_extended r%lu at cf+0x%lx\n",
-		 (long) regnum, (long) (val * dfi->data_align));
+		 (long) regnum, (long) (val * dci->data_align));
 	  break;
 
 	case DW_CFA_offset_extended_sf:
 	  if (((ret = read_regnum (as, a, addr, &regnum, arg)) < 0)
 	      || ((ret = dwarf_read_sleb128 (as, a, addr, &val, arg)) < 0))
 	    goto fail;
-	  set_reg (sr, regnum, DWARF_WHERE_CFAREL, val * dfi->data_align);
+	  set_reg (sr, regnum, DWARF_WHERE_CFAREL, val * dci->data_align);
 	  Debug (15, "CFA_offset_extended_sf r%lu at cf+0x%lx\n",
-		 (long) regnum, (long) (val * dfi->data_align));
+		 (long) regnum, (long) (val * dci->data_align));
 	  break;
 
 	case DW_CFA_restore:
@@ -197,7 +196,7 @@ run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 	  break;
 
 	case DW_CFA_set_loc:
-	  fde_encoding = dfi->flags & UNW_DYN_DFI_FLAG_FDE_PE_MASK;
+	  fde_encoding = dci->fde_encoding;
 	  if ((ret = dwarf_read_encoded_pointer (as, a, addr, fde_encoding,
 						 &c->pi, &curr_ip,
 						 arg)) < 0)
@@ -271,9 +270,9 @@ run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 	    goto fail;
 	  set_reg (sr, DWARF_CFA_REG_COLUMN, DWARF_WHERE_REG, regnum);
 	  set_reg (sr, DWARF_CFA_OFF_COLUMN, 0,
-		   val * dfi->data_align);		/* factored! */
+		   val * dci->data_align);		/* factored! */
 	  Debug (15, "CFA_def_cfa_sf r%lu+0x%lx\n",
-		 (long) regnum, (long) (val * dfi->data_align));
+		 (long) regnum, (long) (val * dci->data_align));
 	  break;
 
 	case DW_CFA_def_cfa_register:
@@ -287,16 +286,16 @@ run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 	  if ((ret = dwarf_read_uleb128 (as, a, addr, &val, arg)) < 0)
 	    goto fail;
 	  set_reg (sr, DWARF_CFA_OFF_COLUMN, 0, val);	/* NOT factored! */
-	  Debug (15, "CFA_def_cfa_offsets 0x%lx\n", (long) val);
+	  Debug (15, "CFA_def_cfa_offset 0x%lx\n", (long) val);
 	  break;
 
 	case DW_CFA_def_cfa_offset_sf:
 	  if ((ret = dwarf_read_sleb128 (as, a, addr, &val, arg)) < 0)
 	    goto fail;
 	  set_reg (sr, DWARF_CFA_OFF_COLUMN, 0,
-		   val * dfi->data_align);	/* factored! */
-	  Debug (15, "CFA_def_cfa_offsets_sf 0x%lx\n",
-		 (long) (val * dfi->data_align));
+		   val * dci->data_align);	/* factored! */
+	  Debug (15, "CFA_def_cfa_offset_sf 0x%lx\n",
+		 (long) (val * dci->data_align));
 	  break;
 
 	case DW_CFA_def_cfa_expression:
@@ -311,7 +310,7 @@ run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 	  *addr += len;
 	  break;
 
-	case DW_CFA_CFA_expression:
+	case DW_CFA_expression:
 	  if ((ret = read_regnum (as, a, addr, &regnum, arg)) < 0)
 	    goto fail;
 
@@ -340,9 +339,9 @@ run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 	  if (((ret = read_regnum (as, a, addr, &regnum, arg)) < 0)
 	      || ((ret = dwarf_read_uleb128 (as, a, addr, &val, arg)) < 0))
 	    goto fail;
-	  set_reg (sr, regnum, DWARF_WHERE_CFAREL, -(val * dfi->data_align));
-	  Debug (15, "CFA_GNU_negative_offsets_extended cfa+0x%lx\n",
-		 (long) -(val * dfi->data_align));
+	  set_reg (sr, regnum, DWARF_WHERE_CFAREL, -(val * dci->data_align));
+	  Debug (15, "CFA_GNU_negative_offset_extended cfa+0x%lx\n",
+		 (long) -(val * dci->data_align));
 	  break;
 
 	case DW_CFA_GNU_window_save:
@@ -359,7 +358,7 @@ run_cfi_program (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 #endif
 	case DW_CFA_lo_user:
 	case DW_CFA_hi_user:
-	  Debug (1, "Unexpected CFA opcode 0x%x", op);
+	  Debug (1, "Unexpected CFA opcode 0x%x\n", op);
 	  ret = -UNW_EINVAL;
 	  goto fail;
 	}
@@ -382,9 +381,13 @@ fetch_proc_info (struct dwarf_cursor *c, unw_word_t ip, int need_unwind_info)
 {
   int ret, dynamic = 1;
 
+  --ip;
+
   if (c->pi_valid && !need_unwind_info)
     return 0;
 
+  memset (&c->pi, 0, sizeof (c->pi));
+
   /* check dynamic info first --- it overrides everything else */
   ret = unwi_find_dynamic_proc_info (c->as, ip, &c->pi, need_unwind_info,
 				     c->as_arg);
@@ -419,32 +422,212 @@ put_unwind_info (struct dwarf_cursor *c, unw_proc_info_t *pi)
 
   if (c->pi_is_dynamic)
     unwi_put_dynamic_unwind_info (c->as, pi, c->as_arg);
+  else if (pi->unwind_info);
+    {
+      mempool_free (&dwarf_cie_info_pool, pi->unwind_info);
+      pi->unwind_info = NULL;
+    }
 }
 
 static inline int
 parse_fde (struct dwarf_cursor *c, unw_word_t ip, dwarf_state_record_t *sr)
 {
-  unw_dyn_dwarf_fde_info_t *dfi;
+  struct dwarf_cie_info *dci;
   unw_word_t addr;
   int ret;
 
-  dfi = c->pi.unwind_info;
-  c->ret_addr_column = dfi->ret_addr_column;
+  dci = c->pi.unwind_info;
+  c->ret_addr_column = dci->ret_addr_column;
 
-  addr = dfi->cie_instr_start;
+  addr = dci->cie_instr_start;
   if ((ret = run_cfi_program (c, sr, ~(unw_word_t) 0, &addr,
-			      dfi->cie_instr_end, dfi)) < 0)
+			      dci->cie_instr_end, dci)) < 0)
     return ret;
 
   memcpy (&sr->rs_initial, &sr->rs_current, sizeof (sr->rs_initial));
 
-  addr = dfi->fde_instr_start;
-  if ((ret = run_cfi_program (c, sr, ip, &addr, dfi->fde_instr_end, dfi)) < 0)
+  addr = dci->fde_instr_start;
+  if ((ret = run_cfi_program (c, sr, ip, &addr, dci->fde_instr_end, dci)) < 0)
     return ret;
 
   return 0;
 }
 
+static inline void
+flush_rs_cache (struct dwarf_rs_cache *cache)
+{
+  int i;
+
+  cache->lru_head = DWARF_UNW_CACHE_SIZE - 1;
+  cache->lru_tail = 0;
+
+  for (i = 0; i < DWARF_UNW_CACHE_SIZE; ++i)
+    {
+      if (i > 0)
+	cache->buckets[i].lru_chain = (i - 1);
+      cache->buckets[i].coll_chain = -1;
+      cache->buckets[i].ip = 0;
+    }
+  for (i = 0; i<DWARF_UNW_HASH_SIZE; ++i)
+    cache->hash[i] = -1;
+}
+
+static inline struct dwarf_rs_cache *
+get_rs_cache (unw_addr_space_t as, intrmask_t *saved_maskp)
+{
+  struct dwarf_rs_cache *cache = &as->global_cache;
+  unw_caching_policy_t caching = as->caching_policy;
+
+  if (caching == UNW_CACHE_NONE)
+    return NULL;
+
+#ifdef HAVE_ATOMIC_H
+  if (!spin_trylock_irqsave (&cache->busy, *saved_maskp))
+    return NULL;
+#else
+# ifdef HAVE_ATOMIC_OPS_H
+  if (AO_test_and_set (&cache->busy) == AO_TS_SET)
+    return NULL;
+# else
+  sigprocmask (SIG_SETMASK, &unwi_full_mask, saved_maskp);
+  if (likely (caching == UNW_CACHE_GLOBAL))
+    {
+      Debug (16, "%s: acquiring lock\n", __FUNCTION__);
+      mutex_lock (&cache->lock);
+    }
+# endif
+#endif
+
+  if (atomic_read (&as->cache_generation) != atomic_read (&cache->generation))
+    {
+      flush_rs_cache (cache);
+      cache->generation = as->cache_generation;
+    }
+
+  return cache;
+}
+
+static inline void
+put_rs_cache (unw_addr_space_t as, struct dwarf_rs_cache *cache,
+		  intrmask_t *saved_maskp)
+{
+  assert (as->caching_policy != UNW_CACHE_NONE);
+
+  Debug (16, "unmasking signals/interrupts and releasing lock\n");
+#ifdef HAVE_ATOMIC_H
+  spin_unlock_irqrestore (&cache->busy, *saved_maskp);
+#else
+# ifdef HAVE_ATOMIC_OPS_H
+  AO_CLEAR (&cache->busy);
+# else
+  if (likely (as->caching_policy == UNW_CACHE_GLOBAL))
+    mutex_unlock (&cache->lock);
+  sigprocmask (SIG_SETMASK, saved_maskp, NULL);
+# endif
+#endif
+}
+
+static inline unw_hash_index_t
+hash (unw_word_t ip)
+{
+  /* based on (sqrt(5)/2-1)*2^64 */
+# define magic	((unw_word_t) 0x9e3779b97f4a7c16ULL)
+
+  return ip * magic >> ((sizeof(unw_word_t) * 8) - DWARF_LOG_UNW_HASH_SIZE);
+}
+
+static inline long
+cache_match (dwarf_reg_state_t *rs, unw_word_t ip)
+{
+  if (ip == rs->ip)
+    return 1;
+  return 0;
+}
+
+static dwarf_reg_state_t *
+rs_lookup (struct dwarf_rs_cache *cache, struct dwarf_cursor *c)
+{
+  dwarf_reg_state_t *rs = cache->buckets + c->hint;
+  unsigned short index;
+  unw_word_t ip;
+
+  ip = c->ip;
+
+  if (cache_match (rs, ip))
+    return rs;
+
+  index = cache->hash[hash (ip)];
+  if (index >= DWARF_UNW_CACHE_SIZE)
+    return 0;
+
+  rs = cache->buckets + index;
+  while (1)
+    {
+      if (cache_match (rs, ip))
+        {
+          /* update hint; no locking needed: single-word writes are atomic */
+          c->hint = cache->buckets[c->prev_rs].hint =
+            (rs - cache->buckets);
+          return rs;
+        }
+      if (rs->coll_chain >= DWARF_UNW_HASH_SIZE)
+        return 0;
+      rs = cache->buckets + rs->coll_chain;
+    }
+}
+
+static inline dwarf_reg_state_t *
+rs_new (struct dwarf_rs_cache *cache, struct dwarf_cursor * c)
+{
+  dwarf_reg_state_t *rs, *prev, *tmp;
+  unw_hash_index_t index;
+  unsigned short head;
+
+  head = cache->lru_head;
+  rs = cache->buckets + head;
+  cache->lru_head = rs->lru_chain;
+
+  /* re-insert rs at the tail of the LRU chain: */
+  cache->buckets[cache->lru_tail].lru_chain = head;
+  cache->lru_tail = head;
+
+  /* remove the old rs from the hash table (if it's there): */
+  if (rs->ip)
+    {
+      index = hash (rs->ip);
+      tmp = cache->buckets + cache->hash[index];
+      prev = 0;
+      while (1)
+	{
+	  if (tmp == rs)
+	    {
+	      if (prev)
+		prev->coll_chain = tmp->coll_chain;
+	      else
+		cache->hash[index] = tmp->coll_chain;
+	      break;
+	    }
+	  else
+	    prev = tmp;
+	  if (tmp->coll_chain >= DWARF_UNW_CACHE_SIZE)
+	    /* old rs wasn't in the hash-table */
+	    break;
+	  tmp = cache->buckets + tmp->coll_chain;
+	}
+    }
+
+  /* enter new rs in the hash table */
+  index = hash (c->ip);
+  rs->coll_chain = cache->hash[index];
+  cache->hash[index] = rs - cache->buckets;
+
+  rs->hint = 0;
+  rs->ip = c->ip;
+  rs->ret_addr_column = c->ret_addr_column;
+
+  return rs;
+}
+
 static int
 create_state_record_for (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 			 unw_word_t ip)
@@ -453,12 +636,14 @@ create_state_record_for (struct dwarf_cursor *c, dwarf_state_record_t *sr,
 
   assert (c->pi_valid);
 
-  for (i = 0; i < DWARF_NUM_PRESERVED_REGS; ++i)
+  memset (sr, 0, sizeof (*sr));
+  for (i = 0; i < DWARF_NUM_PRESERVED_REGS + 2; ++i)
     set_reg (sr, i, DWARF_WHERE_SAME, 0);
 
   switch (c->pi.format)
     {
-    case UNW_INFO_FORMAT_DWARF_FDE:
+    case UNW_INFO_FORMAT_TABLE:
+    case UNW_INFO_FORMAT_REMOTE_TABLE:
       ret = parse_fde (c, ip, sr);
       break;
 
@@ -466,8 +651,6 @@ create_state_record_for (struct dwarf_cursor *c, dwarf_state_record_t *sr,
       ret = parse_dynamic (c, ip, sr);
       break;
 
-    case UNW_INFO_FORMAT_REMOTE_TABLE:
-    case UNW_INFO_FORMAT_TABLE:
     default:
       Debug (1, "Unexpected unwind-info format %d\n", c->pi.format);
       ret = -UNW_EINVAL;
@@ -502,20 +685,22 @@ eval_location_expr (struct dwarf_cursor *c, unw_addr_space_t as,
 static int
 apply_reg_state (struct dwarf_cursor *c, struct dwarf_reg_state *rs)
 {
-  unw_word_t regnum, addr, cfa;
+  unw_word_t regnum, addr, cfa, ip;
+  unw_word_t prev_ip, prev_cfa;
   unw_addr_space_t as;
   dwarf_loc_t cfa_loc;
   unw_accessors_t *a;
   int i, ret;
   void *arg;
 
+  prev_ip = c->ip;
+  prev_cfa = c->cfa;
+
   as = c->as;
   arg = c->as_arg;
   a = unw_get_accessors (as);
 
-  c->cfa_is_sp = 0;
-
-  /* Evaluate the CFA first, because it may be referred to be other
+  /* Evaluate the CFA first, because it may be referred to by other
      expressions.  */
 
   if (rs->reg[DWARF_CFA_REG_COLUMN].where == DWARF_WHERE_REG)
@@ -527,10 +712,7 @@ apply_reg_state (struct dwarf_cursor *c, struct dwarf_reg_state *rs)
 	 the stack-pointer as well.  */
       if ((rs->reg[DWARF_CFA_REG_COLUMN].val == UNW_TDEP_SP)
 	  && (rs->reg[UNW_TDEP_SP].where == DWARF_WHERE_SAME))
-	{
 	  cfa = c->cfa;
-	  c->cfa_is_sp = 1;
-	}
       else
 	{
 	  regnum = dwarf_to_unw_regnum (rs->reg[DWARF_CFA_REG_COLUMN].val);
@@ -548,10 +730,11 @@ apply_reg_state (struct dwarf_cursor *c, struct dwarf_reg_state *rs)
       addr = rs->reg[DWARF_CFA_REG_COLUMN].val;
       if ((ret = eval_location_expr (c, as, a, addr, &cfa_loc, arg)) < 0)
 	return ret;
-      if ((ret = dwarf_get (c, cfa_loc, &cfa)) < 0)
-	return ret;
+      /* the returned location better be a memory location... */
+      if (DWARF_IS_REG_LOC (cfa_loc))
+	return -UNW_EBADFRAME;
+      cfa = DWARF_GET_LOC (cfa_loc);
     }
-  c->cfa = cfa;
 
   for (i = 0; i < DWARF_NUM_PRESERVED_REGS; ++i)
     {
@@ -579,11 +762,24 @@ apply_reg_state (struct dwarf_cursor *c, struct dwarf_reg_state *rs)
 	  break;
 	}
     }
+  c->cfa = cfa;
+  ret = dwarf_get (c, c->loc[c->ret_addr_column], &ip);
+  if (ret < 0)
+    return ret;
+  c->ip = ip;
+  /* XXX: check for ip to be code_aligned */
+
+  if (c->ip == prev_ip && c->cfa == prev_cfa)
+    {
+      Dprintf ("%s: ip and cfa unchanged; stopping here (ip=0x%lx)\n",
+	       __FUNCTION__, (long) c->ip);
+      return -UNW_EBADFRAME;
+    }
   return 0;
 }
 
-HIDDEN int
-dwarf_find_save_locs (struct dwarf_cursor *c)
+static int
+uncached_dwarf_find_save_locs (struct dwarf_cursor *c)
 {
   dwarf_state_record_t sr;
   int ret;
@@ -601,6 +797,69 @@ dwarf_find_save_locs (struct dwarf_cursor *c)
   return 0;
 }
 
+/* The function finds the saved locations and applies the register
+   state as well. */
+HIDDEN int
+dwarf_find_save_locs (struct dwarf_cursor *c)
+{
+  dwarf_state_record_t sr;
+  dwarf_reg_state_t *rs, *rs1;
+  struct dwarf_rs_cache *cache;
+  int ret = 0;
+  intrmask_t saved_mask;
+
+  if (c->as->caching_policy == UNW_CACHE_NONE)
+    return uncached_dwarf_find_save_locs (c);
+
+  cache = get_rs_cache(c->as, &saved_mask);
+  if (!cache)
+    return -UNW_ENOINFO;	/* cache is busy */
+  rs = rs_lookup(cache, c);
+
+  if (rs)
+    {
+      c->ret_addr_column = rs->ret_addr_column;
+      goto apply;
+    }
+
+  if ((ret = fetch_proc_info (c, c->ip, 1)) < 0)
+    goto out;
+
+  if ((ret = create_state_record_for (c, &sr, c->ip)) < 0)
+    goto out;
+
+  rs1 = &sr.rs_current;
+  if (rs1)
+    {
+      rs = rs_new (cache, c);
+      memcpy(rs, rs1, offsetof(struct dwarf_reg_state, ip));
+      if (!rs)
+        {
+          Dprintf ("%s: failed to create unwind rs\n", __FUNCTION__);
+          ret = -UNW_EUNSPEC;
+	  goto out;
+        }
+    }
+  cache->buckets[c->prev_rs].hint = rs - cache->buckets;
+
+  c->hint = rs->hint;
+  c->prev_rs = rs - cache->buckets;
+
+  put_unwind_info (c, &c->pi);
+  ret = apply_reg_state (c, rs);
+
+out:
+  put_rs_cache (c->as, cache, &saved_mask);
+  return ret;
+
+apply:
+  put_rs_cache (c->as, cache, &saved_mask);
+  if ((ret = apply_reg_state (c, rs)) < 0)
+    return ret;
+
+  return 0;
+}
+
 /* The proc-info must be valid for IP before this routine can be
    called.  */
 HIDDEN int
