@@ -1,6 +1,7 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2003 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+   Copyright (C) 2003-2005 Hewlett-Packard Co
+   Copyright (C) 2007 David Mosberger-Tang
+	Contributed by David Mosberger-Tang <dmosberger@gmail.com>
 
 This file is part of libunwind.
 
@@ -24,15 +25,11 @@ OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
 #include <stdio.h>
-#include <string.h>
-#include <unistd.h>
 
-#include <sys/types.h>
+#include "libunwind_i.h"
 
-#include "tdep.h"
-
-extern HIDDEN int
-elfW (valid_object) (struct elf_image *ei)
+HIDDEN int
+elf_w (valid_object) (struct elf_image *ei)
 {
   if (ei->size <= EI_CLASS)
     return 0;
@@ -43,33 +40,33 @@ elfW (valid_object) (struct elf_image *ei)
 
 
 static int
-elfW (lookup_symbol) (unw_word_t ip, struct elf_image *ei,
-		      ElfW (Addr) load_offset,
-		      char *buf, size_t buf_len, unw_word_t *offp)
+elf_w (lookup_symbol) (unw_addr_space_t as,
+		       unw_word_t ip, struct elf_image *ei,
+		       Elf_W (Addr) load_offset,
+		       char *buf, size_t buf_len, unw_word_t *offp)
 {
-  size_t syment_size, str_size;
-  ElfW (Ehdr) *ehdr = ei->image;
-  ElfW (Sym) *sym, *symtab, *symtab_end;
-  ElfW (Off) soff, str_soff;
-  ElfW (Shdr) *shdr, *str_shdr;
-  ElfW (Addr) val, min_dist = ~(ElfW (Addr))0;
+  size_t syment_size;
+  Elf_W (Ehdr) *ehdr = ei->image;
+  Elf_W (Sym) *sym, *symtab, *symtab_end;
+  Elf_W (Off) soff, str_soff;
+  Elf_W (Shdr) *shdr, *str_shdr;
+  Elf_W (Addr) val, min_dist = ~(Elf_W (Addr))0;
   int i, ret = 0;
   char *strtab;
 
-  if (!elfW (valid_object) (ei))
+  if (!elf_w (valid_object) (ei))
     return -UNW_ENOINFO;
 
   soff = ehdr->e_shoff;
   if (soff + ehdr->e_shnum * ehdr->e_shentsize > ei->size)
     {
-      debug (1, "%s: section table outside of image? (%lu > %lu)\n",
-	     __FUNCTION__,
+      Debug (1, "section table outside of image? (%lu > %lu)\n",
 	     (unsigned long) (soff + ehdr->e_shnum * ehdr->e_shentsize),
 	     (unsigned long) ei->size);
       return -UNW_ENOINFO;
     }
 
-  shdr = (ElfW (Shdr) *) ((char *) ei->image + soff);
+  shdr = (Elf_W (Shdr) *) ((char *) ei->image + soff);
 
   for (i = 0; i < ehdr->e_shnum; ++i)
     {
@@ -77,43 +74,42 @@ elfW (lookup_symbol) (unw_word_t ip, struct elf_image *ei,
 	{
 	case SHT_SYMTAB:
 	case SHT_DYNSYM:
-	  symtab = (ElfW (Sym) *) ((char *) ei->image + shdr->sh_offset);
-	  symtab_end = (ElfW (Sym) *) ((char *) symtab + shdr->sh_size);
+	  symtab = (Elf_W (Sym) *) ((char *) ei->image + shdr->sh_offset);
+	  symtab_end = (Elf_W (Sym) *) ((char *) symtab + shdr->sh_size);
 	  syment_size = shdr->sh_entsize;
 
 	  str_soff = soff + (shdr->sh_link * ehdr->e_shentsize);
 	  if (str_soff + ehdr->e_shentsize >= ei->size)
 	    {
-	      debug (1, "%s: string table outside of image? (%lu >= %lu)\n",
-		     __FUNCTION__,
+	      Debug (1, "string table outside of image? (%lu >= %lu)\n",
 		     (unsigned long) (str_soff + ehdr->e_shentsize),
 		     (unsigned long) ei->size);
 	      break;
 	    }
-	  str_shdr = (ElfW (Shdr) *) ((char *) ei->image + str_soff);
-	  str_size = str_shdr->sh_size;
+	  str_shdr = (Elf_W (Shdr) *) ((char *) ei->image + str_soff);
 	  strtab = (char *) ei->image + str_shdr->sh_offset;
 
-	  debug (160, "symtab=0x%lx[%d], strtab=0x%lx\n",
+	  Debug (16, "symtab=0x%lx[%d], strtab=0x%lx\n",
 		 (long) shdr->sh_offset, shdr->sh_type,
 		 (long) str_shdr->sh_offset);
 
 	  for (sym = symtab;
 	       sym < symtab_end;
-	       sym = (ElfW (Sym) *) ((char *) sym + syment_size))
+	       sym = (Elf_W (Sym) *) ((char *) sym + syment_size))
 	    {
-	      if (ELFW (ST_TYPE) (sym->st_info) == STT_FUNC
+	      if (ELF_W (ST_TYPE) (sym->st_info) == STT_FUNC
 		  && sym->st_shndx != SHN_UNDEF)
 		{
-		  val = sym->st_value;
+		  if (tdep_get_func_addr (as, sym->st_value, &val) < 0)
+		    continue;
 		  if (sym->st_shndx != SHN_ABS)
 		    val += load_offset;
-		  debug (160, "0x%016lx info=0x%02x %s\n",
+		  Debug (16, "0x%016lx info=0x%02x %s\n",
 			 (long) val, sym->st_info, strtab + sym->st_name);
 
-		  if ((ElfW (Addr)) (ip - val) < min_dist)
+		  if ((Elf_W (Addr)) (ip - val) < min_dist)
 		    {
-		      min_dist = (ElfW (Addr)) (ip - val);
+		      min_dist = (Elf_W (Addr)) (ip - val);
 		      strncpy (buf, strtab + sym->st_name, buf_len);
 		      buf[buf_len - 1] = '\0';
 		      if (strlen (strtab + sym->st_name) >= buf_len)
@@ -126,7 +122,7 @@ elfW (lookup_symbol) (unw_word_t ip, struct elf_image *ei,
 	default:
 	  break;
 	}
-      shdr = (ElfW (Shdr) *) (((char *) shdr) + ehdr->e_shentsize);
+      shdr = (Elf_W (Shdr) *) (((char *) shdr) + ehdr->e_shentsize);
     }
   if (min_dist >= ei->size)
     return -UNW_ENOINFO;		/* not found */
@@ -141,14 +137,14 @@ elfW (lookup_symbol) (unw_word_t ip, struct elf_image *ei,
    sensitive to the performance of this routine, why bother...  */
 
 HIDDEN int
-elfW (get_proc_name) (pid_t pid, unw_word_t ip, char *buf, size_t buf_len,
-		      unw_word_t *offp)
+elf_w (get_proc_name) (unw_addr_space_t as, pid_t pid, unw_word_t ip,
+		       char *buf, size_t buf_len, unw_word_t *offp)
 {
   unsigned long segbase, mapoff;
-  ElfW (Addr) load_offset = 0;
+  Elf_W (Addr) load_offset = 0;
   struct elf_image ei;
-  ElfW (Ehdr) *ehdr;
-  ElfW (Phdr) *phdr;
+  Elf_W (Ehdr) *ehdr;
+  Elf_W (Phdr) *phdr;
   int i, ret;
 
   ret = tdep_get_elf_image (&ei, pid, ip, &segbase, &mapoff);
@@ -156,7 +152,7 @@ elfW (get_proc_name) (pid_t pid, unw_word_t ip, char *buf, size_t buf_len,
     return ret;
 
   ehdr = ei.image;
-  phdr = (ElfW (Phdr) *) ((char *) ei.image + ehdr->e_phoff);
+  phdr = (Elf_W (Phdr) *) ((char *) ei.image + ehdr->e_phoff);
 
   for (i = 0; i < ehdr->e_phnum; ++i)
     if (phdr[i].p_type == PT_LOAD && phdr[i].p_offset == mapoff)
@@ -165,7 +161,7 @@ elfW (get_proc_name) (pid_t pid, unw_word_t ip, char *buf, size_t buf_len,
 	break;
       }
 
-  ret = elfW (lookup_symbol) (ip, &ei, load_offset, buf, buf_len, offp);
+  ret = elf_w (lookup_symbol) (as, ip, &ei, load_offset, buf, buf_len, offp);
 
   munmap (ei.image, ei.size);
   ei.image = NULL;
