@@ -1,6 +1,7 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2003 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+   Copyright (C) 2003, 2005 Hewlett-Packard Co
+   Copyright (C) 2007 David Mosberger-Tang
+	Contributed by David Mosberger-Tang <dmosberger@gmail.com>
 
 This file is part of libunwind.
 
@@ -30,16 +31,16 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 #include <sys/mman.h>
 #include <sys/stat.h>
 
-#include "internal.h"
+#include "libunwind_i.h"
 
 #if ELF_CLASS == ELFCLASS32
-# define ELFW(x)	ELF32_##x
-# define ElfW(x)	Elf32_##x
-# define elfW(x)	_Uelf32_##x
+# define ELF_W(x)	ELF32_##x
+# define Elf_W(x)	Elf32_##x
+# define elf_w(x)	_Uelf32_##x
 #else
-# define ELFW(x)	ELF64_##x
-# define ElfW(x)	Elf64_##x
-# define elfW(x)	_Uelf64_##x
+# define ELF_W(x)	ELF64_##x
+# define Elf_W(x)	Elf64_##x
+# define elf_w(x)	_Uelf64_##x
 #endif
 
 static inline int
@@ -67,7 +68,8 @@ elf_map_image (struct elf_image *ei, const char *path)
   return 0;
 }
 
-extern HIDDEN int elfW (valid_object) (struct elf_image *ei);
-extern HIDDEN int elfW (get_proc_name) (pid_t pid, unw_word_t ip,
-					char *buf, size_t len,
-					unw_word_t *offp);
+extern int elf_w (valid_object) (struct elf_image *ei);
+extern int elf_w (get_proc_name) (unw_addr_space_t as,
+				  pid_t pid, unw_word_t ip,
+				  char *buf, size_t len,
+				  unw_word_t *offp);
