@@ -1,6 +1,7 @@
 /* libunwind - a platform-independent unwind library
    Copyright (C) 2003-2004 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+   Copyright (C) 2007 David Mosberger-Tang
+	Contributed by David Mosberger-Tang <dmosberger@gmail.com>
 
 This file is part of libunwind.
 
@@ -71,6 +72,7 @@ maps_init (struct map_iterator *mi, pid_t pid)
 
   mi->fd = open (path, O_RDONLY);
   mi->offset = 0;
+  mi->buf_size = 0;
 
   cp = NULL;
   if (mi->fd >= 0)
@@ -139,9 +141,12 @@ scan_dec (char *cp, unsigned long *valp)
 
   while (1)
     {
-      digit = *cp++;
+      digit = *cp;
       if ((digit - '0') <= 9)
-	digit -= '0';
+	{
+	  digit -= '0';
+	  ++cp;
+	}
       else
 	break;
       val = (10 * val) + digit;
@@ -159,7 +164,11 @@ scan_char (char *cp, char *valp)
   if (!cp)
     return NULL;
 
-  *valp = *cp++;
+  *valp = *cp;
+
+  /* don't step over NUL terminator */
+  if (*cp)
+    ++cp;
   return cp;
 }
 
@@ -219,13 +228,22 @@ maps_next (struct map_iterator *mi,
 	    {
 	      /* copy down the remaining bytes, if any */
 	      if (bytes_left > 0)
-		memcpy (mi->buf_end - mi->buf_size, mi->buf, bytes_left);
+		memmove (mi->buf_end - mi->buf_size, mi->buf, bytes_left);
 
 	      mi->buf = mi->buf_end - mi->buf_size;
 	      nread = read (mi->fd, mi->buf + bytes_left,
 			    mi->buf_size - bytes_left);
 	      if (nread <= 0)
 		return 0;
+	      else if ((size_t) (nread + bytes_left) < mi->buf_size)
+		{
+		  /* Move contents to the end of the buffer so we
+		     maintain the invariant that all bytes between
+		     mi->buf and mi->buf_end are valid.  */
+		  memmove (mi->buf_end - nread - bytes_left, mi->buf,
+			  nread + bytes_left);
+		  mi->buf = mi->buf_end - nread - bytes_left;
+		}
 
 	      eol = mi->buf + bytes_left + nread - 1;
 
